//typedef	float		real;		//  in single precision
typedef	double	real;		//  in double precision

const real	RADIUS	=6371.0;
const real	DEPMOHO	=24.37109;
const real	DEP220	=220.0;
const real	DEP400	=400.0;
const real	DEP670	=670.0;
const real	DEPCMB	=2891.04;
const real	DEPICB	=5149.51;
const real	PI		=3.14159265;
const real	RAD		=0.017453292519943;	// (pi/180.0)
const  real	HPI		=0.5*PI;		// 0.5*pi
const  real	TWOPI		=2.0*PI;		// 0.5*pi
const int MAXLAYER = 10;

enum Boolean {FALSE,TRUE};
		
template <class Type>
inline Type min (Type a, Type b){
   return a<b ? a : b;
}

template <class Type>
inline Type max (Type a, Type b){
   return a>b ? a : b;
}

inline void swap(real *a, real *b)
{
real c;
   c=*a;
   *a=*b;
   *b=c;
}

class Geographic{
friend class Spherical2;
friend class Cartesian;
friend class EqSt;
public:
  void SetValue(real xlat, real xlon){lat=xlat; lon=xlon;}
  real lat,lon;
};

class Cartesian{
friend class Spherical2;
friend class Geographic;
friend class TransMatrix;
friend class EqSt;
public:
   void Normalize();
   void ConvFromSph2(class Spherical2&);
   Cartesian OutProdWith(class Cartesian&);
private:
   real x,y,z;
};

class Spherical2{
friend class Cartesian;
friend class Geographic;
friend class TransMatrix;
friend class EqSt;
public:
  Spherical2 ForwardTrans(real *);
  Spherical2 InverseTrans(real *);
  void ConvFromGeo(Geographic);
  real theta,phi;
};

class TransMatrix{
friend class Spherical2;
friend class EqSt;
public:
  void SetFromVec(class Cartesian&, class Cartesian&, class Cartesian&);
  real matrix[9];
};


class EqSt{			// earthquake and station
friend void ReadArow(int,class EqSt *,float *,short *,int*,int*);
friend void CapReadArow(int,class EqSt *,float *,short *,int*,int*);
// the following routine reads everything, including lat, lon, etc
friend void  AddArow(class EqSt &,int &,float *, short *, float *, float *, float, float, float, 
		int, int, int, int);
friend void AddArow_Resolution(class EqSt &,int &,float *, short *, float *, float *, float, int);
public:
   void CalEqStDist();
   real GetDist() const { return dist;}
   real GetEqdep() const { return eqdep;}
   void ReadEq();
   void ReadSt();
   void CalTrans4();
   TransMatrix GetTrans4() const { return trans;}
   void ForwardTrans4(const int &);
   void InverseTrans4(const int &);
   void InvTransRPhi(const real*, const real*, const int&);
   void Init(int);
   int  numpt;
   real *r,*theta,*phi;	// to save the raypath between eq and st
   real *dl;		// ray path lengths
// below are used for topography data
   short *indsurf;
   float  *surfbs;
   int   nsurf;
private:
   int  ist;		// station index
   int  neq;		// earthquake index
   real dist;		// distance between eq and st (in radian)
   real eqdep;		// earthquake depth
   real stelev;		// station elevation (in km)
   real res1,res2;	// observed and predicted residuals
   real time;		// Prem predicted travel time
   Geographic geq,gst;	// geographic location of eq and st
   Spherical2 seq,sst;	// spherical coordinate of eq and station
   Cartesian ceq,cst;	// eq and st in Cartesian coordinates
   TransMatrix trans;	// transformation matrix
   Spherical2 eqnew,stnew;
};
const int	ST		=1;
const int	EQ		=2;
const int	EQST		=3;



enum MDCHOICE{MODALL,KNOTS,REGIONAL,SPLIT,SPLIT_REGIONAL, MULTISPLIT};
enum REGION{WHOLEMANTLE,UPPERMANTLE,LOWERMANTLE,UPLOWMANTLE};
class Model{
	public:
        virtual void ReadModel(int)=0;
        int  GetRegion() const { return region;}
        int  GetMaxK() const { return maxk;}
	protected:
	  int maxk;		// order of radial functions or # of B-splines
	  int maxk_um, maxk_lm; // radial orders for split model
	  int region;		// the region for which model is defined
	  real rtop,rmid,rbom;
	  real *coeff;		// model coefficients
};

class BSModel: public Model{
      friend void  WriteIpm(BSModel&, float *, float *, int &, int, int);
      friend real  SumArow(BSModel &l,float *,short *,int &);
      public:
        void ReadModel(int choice=MODALL);
        void ReadModel_split(int choice);
        void ReadModel_multisplit(int choice=MODALL, char *filename="s12wm13_multispilt.XBS");
	void Read1stLine(fstream &inFile, int choice);
        void InitKnots(int);
        void CalAvgDist();
        real GetAvgDist() const { return avgdist;}
        real DistBt2(const int&, const int&);
        int  GetHPts() const { return gpt;}
	int  GetNumSplit() const {return nsplit_lm+nsplit_um;}
      private:
        int gpt;		// number of horizontal grid points
        real *theta,*phi;
        real *rknt;	// knots expressed as depths
        real avgdist;	// average distance between grid points
        int *ajknts; 	// to save adjacent points
        real *coeff; 	// model coefficients
        char modelname[132];
// the following are useful for multiply split model
	int  nsplit_um;
	int  nsplit_lm;
	real **rknt_mult; // for multiply split B-spline model
	int  nbsperlayer[MAXLAYER];
};

#define between(x,y,z)     (((x)<=(y) && (z)>(y) )|| ((z)<(y) && (x)>=(y)) )

inline real LinearInt(real &,real &,real &,real &,real &);
int GetReg(real,real *,int);
inline real fdelta(real&, real&);

void BsplineFun(int, real, real *, int, real *);
int SysOpen(char *,int);

#define DIST(theta,phi,phi1)	acos(sin(theta)*cos((phi)-(phi1)))
#define PERMS 0666    /* RW for owner, group, others */
void ReadArow(int,class EqSt *, float *,short *,int*, int*);
void  AddArow(class EqSt &,int &,float *, short *, float *, float *, float, float, float, 
		int, int, int, int);
void  AddArow_Resolution(class EqSt &,int &,float *, short *, float *, float *, float, int);
void  ReadArow_Eqinfo(int,float *,short *,int &, int &, int, float &, float &, float &, float &, float &, int);
void RdAmatModPara(int &, int, int &,int &,int &);
real  SumArow(BSModel &l,float *,short *,int &);
void  WriteIpm(BSModel&, float *, float *, int &, int, int);
void CheckMenu(int argc, char *argv[], char &model, char &modeltype);

