/* this library manipulate sac commands for convenient calls from C */

#include <stdio.h>
#include <math.h>
#include <sys/file.h>
#include <unistd.h>
#include <string.h>
#include "sachead.h"

void SacOnOff(FILE *fp, int ifon)
/* ifon:  1=to turn on sac  others=turn off sac) */
{
	if(ifon==1) {
		fprintf(fp, "sac2000 << EOF\n");
	}else{
		fprintf(fp, "quit\n");
		fprintf(fp, "EOF\n");
		fclose(fp);
	}
}

void GenRotScript_simple(FILE *fp, char *name1, char *name2, char *name3, char *name4, float ang) 
{
/* assume two horizontals and rotate through a given angle */
	fprintf(fp, "read %s %s\n", name1, name2, ang);
	fprintf(fp, "rmean\n");
	fprintf(fp, "rotate through %.2f\n", ang);
	fprintf(fp, "write %s %s\n", name3, name4);
}



/* Notes on Sac Rotation:

SYNTAX:
ROTATE {to GCP|TO v|THROUGH v|,{NORMAL|REVERSED}

INPUT:
      TO GCP  :  Rotate to the "great circle path".  Both components must be
          horizontals.  The station and event coordinates header fields must
          be defined.  
      TO v  :  Rotate to the angle v in degrees.  Both components must be
          horizontals.  
      THROUGH v  :  Rotate through the angle v in degrees.  One component may
          be vertical.  
      NORMAL  :  Output components with normal polarity.  
      REVERSED  :  Output components with reversed polarity.
DEFAULT VALUES:
ROTATE TO GCP NORMAL

DESCRIPTION:
      Pairs of data components are rotated in this command.  Each pair must
have the same station name, event name, and sampling rate.  In the THROUGH
option both components are simply rotated through the requested angle.  One of
those components may be a vertical.  Rotations in the horizontal plane are
clockwise from north.  Rotations with a vertical component are clockwise from
up.
      Both components must be horizontals when the TO option is used.  This
means that CMPAZ must be definined and that CMPINC must be 90 degrees.  After
the rotation is completed the first component of each pair will be directed
along the angle given after to TO keyword.  If the TO GCP option is used this
component will be directed along the angle given by the station-event back
azimuth plus or minus 180 degrees.  This component therefore points from the
event toward the station.  The station and event coordinates header fields
(STLA, STLO, EVLA, and EVLO) must be defined so that the back azimuth can be
calculated.
      The NORMAL and REVERSED options also apply only to horizontal rotations.
If the NORMAL option is used the second component leads the first by 90 
degrees.  If the REVERSED option is used it lags the first by 90 degrees.

EXAMPLES:
To rotate a pair of horizontals through 123.43 degrees:
       u: READ XYZ.N XYZ.E
       u: ROTATE TO 123.43
To rotate two sets of horizontals to the great circle path: 
      u: READ ABC.N ABC.E DEF.N DEF.E 
      u: ROTATE TO GCP 
      u: W ABC.R ABC.T DEF.R DEF.T
In the above example if the BAZ header variable had been 33 degrees, the
radial components would be at 213 degrees and the tangential components at
303 degrees.  If reversed polarity had been requested the tangential
components would be at 123 degrees.
*/

void GenRotScript_obs(FILE *fp, char *name1, char *name2, char *name3, char *name4, float ang, float fl, float fh, int np) 
{
   void SacFilter(FILE *fp, float fl, float fh, int np);
/* assume two horizontals for OBS data (has no orientations), 
   change the header and rotate through a given angle */
	fprintf(fp, "read %s %s\n", name1, name2);
	fprintf(fp, "write rottemp.R rottemp.T\n");
	fprintf(fp, "read rottemp.R rottemp.T\n");
	fprintf(fp, "rmean\n");
	fprintf(fp, "chnhdr cmpinc 90 \n");
	fprintf(fp, "chnhdr file 1 cmpaz 90 \n");
	fprintf(fp, "chnhdr file 2 cmpaz 0 \n");
	fprintf(fp, "rotate through %.2f\n", ang);
	SacFilter(fp,fl,fh, np);
	fprintf(fp, "write %s %s\n", name3, name4);	
}

void SacRead1Fl(FILE *fp, char *name)
{  /* read 1 file only */
	fprintf(fp, "read %s\n", name);
}

void SacReadMore(FILE *fp, char *name)
{  /* read more files */
	fprintf(fp, "read more %s\n", name);
}

void SacRead2Fl(FILE *fp, char *nameN, char *nameE)
{  /* read two horizontals for rotation, NS component should be 1st */
	fprintf(fp, "read %s %s\n", nameN, nameE);
}

void SacRotToGCP(FILE *fp) 
{  /* keep in mind all rotation are clockwise !! */
	fprintf(fp, "rotate to gcp\n");
}

void SacRotThrough(FILE *fp, float ang) 
{
	fprintf(fp, "rotate through %.2f\n",ang);
}

void SacRotTo(FILE *fp, float ang) 
{
	fprintf(fp, "rotate to %.2f\n",ang);
}

void Gen1File(FILE *fp, char *name)
{
	fprintf(fp, "write %s\n", name);
}

void Gen2File(FILE *fp, char *name1, char *name2)
{
	fprintf(fp, "write %s %s\n", name1, name2);
}

void Gen3File(FILE *fp, char *name1, char *name2, char *name3)
{
	fprintf(fp, "write %s %s %s\n", name1, name2, name3);
}

void GenInterpolate(FILE *fp, float dt)
{  /* interpolate data to a new sampling rate of dt  */
	fprintf(fp, "interpolate delta %.3f\n", dt);
}

void GenXlim(FILE *fp, float xlow, float xhigh)
{ /* change label limits */
	fprintf(fp, "xlim %f %f\n", xlow, xhigh);
}

void SacCut(FILE *fp, float xlow, float xhigh)
{
	fprintf(fp, "cut %f %f\n", xlow, xhigh);
	fprintf(fp, "cut off\n");
}

void ReadData(FILE *fp,char *datanm)
{
 /* read in data file */
    fprintf(fp, "read more %s\n", datanm);
}

void SacFilter(FILE *fp, float fl, float fh, int np)
{
/* script for filtering using buttersworth filter, demean first always */
	printf("inside filtering routine, fl = %f  fh = %f\n", fl, fh);
	fprintf(fp, "bandpass butter npoles 4 corner %.2f %.2f passes %d \n", fl, fh, np);
}

void StartSGF(FILE *fp)
{
/* start the process of outputing what is on the screen */
	fprintf(fp, "bd sgf\n");
}

void SacDmean(FILE *fp)
{
/* remove mean*/
	fprintf(fp, "rmean\n");
}

void SacPlot(FILE *fp)
{
/* script plotting multiple traces on a single plot */
	fprintf(fp, "plot1\n");
}

void SacToPs(FILE *fp, char *outname)
{
/* script for converting .sgf file to postscript file */
	fprintf(fp, "sgftops f001.sgf %s\n", outname);
}


/*
**
**	plotsac
**
**	originally written 4/89 by Scott MacHaffie to read sac binary and writes sac ascii, 
**      I modified it to plot traces and variety of options from the trace using the GMT software,
**	J. Gu, 2002.
*/

void plotsac(char *name,float tselmin,float tselmax,float *xlat,float *xlon,float *slat,float *slon,float *xarray,float *torig,float *samp,int *ndat)
/* tselmin, tselmax; output data range */
/* *xlon, *xlat; earthquake coordinates */
/* *slat, *slon; station coordinates */
/* *xarray, *torig, *samp; array to save seismogram, origin time sampling rate */
/* *ndat;  number of data points within range */	
{
	FILE *inp;		/* input file handle */
	long l;			/* return codes from convert functions */
	long convert_hdr(FILE *inp,float tselmin,float tselmax,float *xlon,float *xlat,
			float *slat,float *slon,float *xarray,float *torig,float *samp,int *ndat);
	int  ifselrange;
	FILE *fp, *fopen();
	struct SAChead2 header;

/* hardwired to deal with 1 sac file only */

	/* input file is opened for fscanf access */
	if((inp = fopen(name, "r")) == NULL) {
		fprintf(stderr, "couldn't open input file\n");
		exit(1);
	}
	if((l = convert_hdr(inp,tselmin,tselmax,xlon,xlat,slat,slon,xarray,torig,samp,ndat)) < 0L) {
		fclose(inp);
		fprintf(stderr, "error converting header\n");
		exit(3);
	}	
	fclose(inp);
}

	

	long convert_hdr(FILE *inp,float tselmin,float tselmax,float *xlon,float *xlat,
			float *slat,float *slon,float *xarray,float *torig,float *samp,int *ndat)
/*
**
**	convert_hdr() -- read header from input file
*/
{
	int i, j;
	long l, l1, l2, l3, l4, l5;
	unsigned long u1, u2, u3, u4, u5;
	float f1, f2, f3, f4, f5;
	struct SAChead2 header;
	long convert_body();	/* convert SAC body */

	if (fread((char *) &header, sizeof(header), 1, inp) != 1) {
	     fprintf(stderr,"Couldn't read header from input\n");
	     return(-1L);
	}
	*samp=header.SACfloat[0];
/*
	printf("time inc = %f  ", header.SACfloat[0]);
	printf("origin = %f\n", header.SACfloat[7]);
	printf("P arrival = %f\n", header.SACfloat[8]);
	printf("S arrival = %f\n", header.SACfloat[10]);
	printf("event lat = %f\n", header.SACfloat[35]);
	printf("event lon = %f\n", header.SACfloat[36]);
	printf("station lat = %f\n", header.SACfloat[31]);
	printf("station lon = %f\n", header.SACfloat[32]);
	printf("S arrival = %f\n", header.SACfloat[38]);
*/
	/* write floating point data -- 5 numbers on a line */
	for (i = 0; i < NUM_FLOAT / 5; i++) {
		f1 = header.SACfloat[i * 5];
		f2 = header.SACfloat[i * 5 + 1];
		f3 = header.SACfloat[i * 5 + 2];
		f4 = header.SACfloat[i * 5 + 3];
		f5 = header.SACfloat[i * 5 + 4];
	}

	for (i = 0; i < MAXINT / 5; i++) {
		l1 = header.SACint[i * 5];
		l2 = header.SACint[i * 5 + 1];
		l3 = header.SACint[i * 5 + 2];
		l4 = header.SACint[i * 5 + 3];
		l5 = header.SACint[i * 5 + 4];
/*
		printf("%10ld %10ld %10ld %10ld %10ld\n", l1, l2, l3, l4, l5);
*/
	}

	u1 = header.SACun[0];
	u2 = header.SACun[1];
	u3 = header.SACun[2];
	u4 = header.SACun[3];
	u5 = header.SACun[4];
/*
	printf("%10lu %10lu %10lu %10lu %10lu\n", u1, u2, u3, u4, u5);
*/

	*xlat=header.SACfloat[35];
	*xlon=header.SACfloat[36];
	*slat=header.SACfloat[31];
	*slon=header.SACfloat[32];
	l=header.SACint[9];
	if ((l = convert_body(inp,l,header,tselmin,tselmax,xarray,torig,ndat)) < 0L) {
		fclose(inp);
		fprintf(stderr, "error converting body\n");
		exit(4);
	}
	/* return the number of data points in the file */
	return(header.SACint[9]);
}

long convert_body(inp,num,header,tselmin,tselmax,xarray,torig,ndat)
/*
**
**	convert_body() -- convert the floating point data from binary to ascii
**
*/
FILE *inp;	/* input file pointer */
long num;	/* number of data points */
struct SAChead2 header;
float tselmin,tselmax;
float *xarray;	/* array to save seismogram and sampling rate */
float *torig;
int   *ndat;		
{
	int i,ifsetorig=0;
	long cnt;		/* number of data points read */
	float f;		/* scratch floating point variable */
	float time;
	struct conv {		/* used to read a line of data at a time */
		float f1;
		float f2;
		float f3;
		float f4;
		float f5;
	} data;
	float ttinc, tt;

	/* write lines of 5 data points */
	ttinc=header.SACfloat[0];
	tt=header.SACfloat[7];  /* origin time */
	i=0;
	for (cnt = 0L; cnt < num - 5; cnt += 5) {
	   if (fread((char *) &data, sizeof(data), 1, inp) != 1) {
		fprintf(stderr, "Couldn't read data from");
		fprintf(stderr, " input\n");
		return(-1L);
	   }
/*
 printf("%d tt=%f, tselmin=%f, tselmax=%f\n", i, tt, tselmin, tselmax);
*/

	   if(tselmin<=tt && tselmax>=tt) {
		if(ifsetorig==0){
			*torig=tt;	
		  	ifsetorig=1;
		}
		xarray[i]=data.f1;
		i++;
	   }
	   tt += ttinc;
	   if(tselmin<=tt && tselmax>=tt) {
		xarray[i]=data.f2;
		i++;
	   }
	   tt += ttinc;
	   if(tselmin<=tt && tselmax>=tt) {
		xarray[i]=data.f3;
		i++;
	   }
	   tt += ttinc;
	   if(tselmin<=tt && tselmax>=tt) {
		xarray[i]=data.f4;
		i++;
	   }
	   tt += ttinc;
	   if(tselmin<=tt && tselmax>=tt) {
		xarray[i]=data.f5;
		i++;
	   }
	   tt += ttinc;
	}

	/* write a line with < 5 data points */
	for ( ; cnt < num; cnt++) {
		if (fread((char *) &f, sizeof(f), 1, inp) != 1) {
			fprintf(stderr, "Couldn't read data from");
			fprintf(stderr, " input\n");
			return(-1L);
		}
		if(tselmin<=tt && tselmax>=tt) {
			xarray[i]=f;
			i++;
		}	
		tt += ttinc;
	}
	*ndat=i;
	return(0L);
}
